@@ -24,13 +24,12 @@ import itertools
 import logging
 import os
 import re
-import shutil
 import subprocess
 import sys
 import tempfile
 import time
 import traceback
-from typing import Optional, List, Sequence, Iterator, Iterable, Tuple
+from typing import Optional, List, Sequence, Iterator, Iterable, Tuple, Union, Callable
 
 from tqdm import trange
 
@@ -42,17 +41,15 @@ from breezy.commit import NullCommitReporter
 from breezy.errors import NoSuchFile
 from breezy.osutils import is_inside
 from breezy.rename_map import RenameMap
-from breezy.transform import revert
 from breezy.tree import Tree
 from breezy.workingtree import WorkingTree
-
-from debian.deb822 import Deb822
+from breezy.workspace import reset_tree, check_clean_tree
 
 
 from debmutate.reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 118)
+__version__ = (0, 126)
 version_string = ".".join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]
 DEFAULT_MINIMUM_CERTAINTY = "certain"
@@ -135,13 +132,6 @@ class NotDebianPackage(Exception):
         super(NotDebianPackage, self).__init__(tree.abspath(path))
 
 
-class PendingChanges(Exception):
-    """The directory has pending changes."""
-
-    def __init__(self, tree, subpath=""):
-        super(PendingChanges, self).__init__(tree.abspath(subpath))
-
-
 class FixerResult(object):
     """Result of a fixer run."""
 
@@ -535,19 +525,19 @@ def read_desc_file(path: str, force_subprocess: bool = False) -> Iterator[Fixer]
     Yields:
       Fixer objects
     """
+    from ruamel.yaml import YAML
+    yaml = YAML()
     dirname = os.path.dirname(path)
     with open(path, "r") as f:
-        for paragraph in Deb822.iter_paragraphs(f):
-            name = os.path.splitext(paragraph["Fix-Script"])[0]
-            script_path = os.path.join(dirname, paragraph["Fix-Script"])
-            if "Lintian-Tags" in paragraph:
-                tags = [tag.strip() for tag in paragraph["Lintian-Tags"].split(",")]
-            else:
-                tags = []
-            if script_path.endswith(".py") and not force_subprocess:
-                yield PythonScriptFixer(name, tags, script_path)
-            else:
-                yield ScriptFixer(name, tags, script_path)
+        data = yaml.load(f)
+    for paragraph in data:
+        name = os.path.splitext(paragraph["script"])[0]
+        script_path = os.path.join(dirname, paragraph["script"])
+        tags = paragraph.get("lintian-tags", [])
+        if script_path.endswith(".py") and not force_subprocess:
+            yield PythonScriptFixer(name, tags, script_path)
+        else:
+            yield ScriptFixer(name, tags, script_path)
 
 
 def select_fixers(
@@ -617,30 +607,6 @@ def increment_version(v: Version) -> None:
         )
 
 
-def delete_items(deletables, dry_run: bool = False):
-    """Delete files in the deletables iterable"""
-
-    def onerror(function, path, excinfo):
-        """Show warning for errors seen by rmtree."""
-        # Handle only permission error while removing files.
-        # Other errors are re-raised.
-        if function is not os.remove or excinfo[1].errno != errno.EACCES:
-            raise
-        logging.warning("unable to remove %s", path)
-
-    for path in deletables:
-        if os.path.isdir(path):
-            shutil.rmtree(path, onerror=onerror)
-        else:
-            try:
-                os.unlink(path)
-            except OSError as e:
-                # We handle only permission error here
-                if e.errno != errno.EACCES:
-                    raise e
-                logging.warning('unable to remove "%s": %s.', path, e.strerror)
-
-
 def get_committer(tree: WorkingTree) -> str:
     """Get the committer string for a tree.
 
@@ -716,36 +682,6 @@ def only_changes_last_changelog_block(
         return str(new_cl) == str(old_cl)
 
 
-def reset_tree(
-    local_tree: WorkingTree,
-    basis_tree: Optional[Tree] = None,
-    subpath: str = "",
-    dirty_tracker=None,
-) -> None:
-    """Reset a tree back to its basis tree.
-
-    This will leave ignored and detritus files alone.
-
-    Args:
-      local_tree: tree to work on
-      dirty_tracker: Optional dirty tracker
-      subpath: Subpath to operate on
-    """
-    if dirty_tracker and not dirty_tracker.is_dirty():
-        return
-    if basis_tree is None:
-        basis_tree = local_tree.branch.basis_tree()
-    revert(local_tree, basis_tree, [subpath] if subpath not in (".", "") else None)
-    deletables: List[str] = []
-    # TODO(jelmer): Use basis tree
-    for p in local_tree.extras():
-        if not is_inside(subpath, p):
-            continue
-        if not local_tree.is_ignored(p):
-            deletables.append(local_tree.abspath(p))
-    delete_items(deletables)
-
-
 def certainty_sufficient(
     actual_certainty: str, minimum_certainty: Optional[str]
 ) -> bool:
@@ -768,46 +704,6 @@ def certainty_sufficient(
     return actual_confidence <= minimum_confidence
 
 
-def check_clean_tree(
-    local_tree: WorkingTree, basis_tree: Tree, subpath: str = ""
-) -> None:
-    """Check that a tree is clean and has no pending changes or unknown files.
-
-    Args:
-      local_tree: The tree to check
-      basis_tree: Tree to check against
-      subpath: Subpath of the tree to check
-    Raises:
-      PendingChanges: When there are pending changes
-    """
-    # Just check there are no changes to begin with
-    changes = local_tree.iter_changes(
-        basis_tree,
-        include_unchanged=False,
-        require_versioned=False,
-        want_unversioned=True,
-        specific_files=[subpath],
-    )
-
-    def relevant(p, t):
-        if not p:
-            return False
-        if not is_inside(subpath, p):
-            return False
-        if t.is_ignored(p):
-            return False
-        if not t.has_versioned_directories() and t.kind(p) == "directory":
-            return False
-        return True
-
-    if any(
-        change
-        for change in changes
-        if relevant(change.path[0], basis_tree) or relevant(change.path[1], local_tree)
-    ):
-        raise PendingChanges(local_tree, subpath)
-
-
 def has_non_debian_changes(changes, subpath):
     for change in changes:
         for path in change.path:
@@ -890,7 +786,7 @@ def run_lintian_fixer(  # noqa: C901
     local_tree: WorkingTree,
     fixer: Fixer,
     committer: Optional[str] = None,
-    update_changelog: Optional[bool] = None,
+    update_changelog: Union[bool, Callable[[], bool]] = True,
     compat_release: Optional[str] = None,
     minimum_certainty: Optional[str] = None,
     trust_package: bool = False,
@@ -1033,25 +929,16 @@ def run_lintian_fixer(  # noqa: C901
 
         summary = "Add patch %s: %s" % (patch_name, summary)
 
-    if update_changelog is None:
-        from .detect_gbp_dch import guess_update_changelog
-
-        dch_guess = guess_update_changelog(
-            local_tree, os.path.join(subpath, "debian"), cl)
-        if dch_guess:
-            update_changelog = dch_guess[0]
-            _note_changelog_policy(update_changelog, dch_guess[1])
-        else:
-            # Assume we should update changelog
-            update_changelog = True
-
-    if update_changelog and only_changes_last_changelog_block(
+    if only_changes_last_changelog_block(
         local_tree, basis_tree, changelog_path, changes
     ):
         # If the script only changed the last entry in the changelog,
         # don't update the changelog
         update_changelog = False
 
+    if callable(update_changelog):
+        update_changelog = update_changelog()
+
     if update_changelog:
         from .changelog import add_changelog_entry
 
@@ -1087,6 +974,7 @@ class ManyResult(object):
         self.failed_fixers = {}
         self.formatting_unpreservable = {}
         self.overridden_lintian_issues = []
+        self.changelog_behaviour = None
 
     def minimum_success_certainty(self) -> str:
         """Return the minimum certainty of any successfully made change."""
@@ -1098,33 +986,53 @@ class ManyResult(object):
             ]
         )
 
-    def __tuple__(self):
-        return (self.success, self.failed_fixers)
-
-    def __iter__(self):
-        return iter(self.__tuple__())
-
 
 def get_dirty_tracker(
     local_tree: WorkingTree, subpath: str = "", use_inotify: bool = None
 ):
     """Create a dirty tracker object."""
     if use_inotify is True:
-        from .dirty_tracker import DirtyTracker
+        from breezy.dirty_tracker import DirtyTracker
 
         return DirtyTracker(local_tree, subpath)
     elif use_inotify is False:
         return None
     else:
         try:
-            from .dirty_tracker import DirtyTracker
+            from breezy.dirty_tracker import DirtyTracker
         except ImportError:
             return None
         else:
             return DirtyTracker(local_tree, subpath)
 
 
-def run_lintian_fixers(
+def determine_update_changelog(local_tree, debian_path):
+    from .detect_gbp_dch import (
+        guess_update_changelog,
+        ChangelogBehaviour,
+        )
+
+    changelog_path = os.path.join(debian_path, 'changelog')
+
+    try:
+        with local_tree.get_file(changelog_path) as f:
+            cl = Changelog(f)
+    except NoSuchFile:
+        # If there's no changelog, then there's nothing to update!
+        return False
+
+    behaviour = guess_update_changelog(local_tree, debian_path, cl)
+    if behaviour:
+        _note_changelog_policy(behaviour.update_changelog, behaviour.explanation)
+    else:
+        # If we can't make an educated guess, assume yes.
+        behaviour = ChangelogBehaviour(
+            True, "Assuming changelog should be updated")
+
+    return behaviour
+
+
+def run_lintian_fixers(  # noqa: C901
     local_tree: WorkingTree,
     fixers: List[Fixer],
     update_changelog: bool = True,
@@ -1168,11 +1076,25 @@ def run_lintian_fixers(
            error that occurred
     """
     basis_tree = local_tree.basis_tree()
-    check_clean_tree(local_tree, basis_tree, subpath)
+    check_clean_tree(local_tree, basis_tree=basis_tree, subpath=subpath)
     fixers = list(fixers)
     dirty_tracker = get_dirty_tracker(
         local_tree, subpath=subpath, use_inotify=use_inotify
     )
+
+    # If we don't know whether to update the changelog, then find out *once*
+    if update_changelog is None:
+        changelog_behaviour = None
+
+        def update_changelog():
+            nonlocal update_changelog, changelog_behaviour
+            changelog_behaviour = determine_update_changelog(
+                local_tree, os.path.join(subpath, "debian"))
+            update_changelog = changelog_behaviour.update_changelog
+            return update_changelog
+    else:
+        changelog_behaviour = None
+
     ret = ManyResult()
     with trange(len(fixers), leave=False, disable=None) as t:
         for i, fixer in enumerate(fixers):
@@ -1247,6 +1169,8 @@ def run_lintian_fixers(
                     )
                 ret.success.append((result, summary))
                 basis_tree = local_tree.basis_tree()
+    if changelog_behaviour:
+        ret.changelog_behaviour = changelog_behaviour
     return ret
 
 
