@@ -31,6 +31,7 @@ import distro_info
 import breezy
 from breezy.branch import Branch
 from breezy.workingtree import WorkingTree
+from breezy.workspace import WorkspaceDirty
 
 from breezy.errors import (  # noqa: E402
     DependencyNotPresent,
@@ -43,7 +44,6 @@ import breezy.bzr  # noqa: E402
 
 from . import (  # noqa: E402
     NotDebianPackage,
-    PendingChanges,
     DescriptionMissing,
     available_lintian_fixers,
     find_fixers_dir,
@@ -149,6 +149,10 @@ def main(argv=None):  # noqa: C901
     fixer_group.add_argument(
         "--uncertain", action="store_true", help="Include changes with lower certainty."
     )
+    fixer_group.add_argument(
+        "--yolo", action="store_true", help=argparse.SUPPRESS
+    )
+
     fixer_group.add_argument(
         "--force-subprocess", action="store_true", default=False, help=argparse.SUPPRESS
     )
@@ -310,7 +314,7 @@ def main(argv=None):  # noqa: C901
             if update_changelog is None:
                 update_changelog = cfg.update_changelog()
         if minimum_certainty is None:
-            if args.uncertain:
+            if args.uncertain or args.yolo:
                 minimum_certainty = "possible"
             else:
                 minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
@@ -344,7 +348,7 @@ def main(argv=None):  # noqa: C901
             except NotDebianPackage:
                 report_fatal("not-debian-package", "Not a Debian package")
                 return 1
-            except PendingChanges:
+            except WorkspaceDirty:
                 logging.error("%s: Please commit pending changes first.", wt.basedir)
                 if args.verbose:
                     from breezy.status import show_tree_status
@@ -433,9 +437,13 @@ def main(argv=None):  # noqa: C901
             failed = {
                 name: str(e)
                 for (name, e) in overall_result.failed_fixers.items()}
+            debian_context = {}
+            if overall_result.changelog_behaviour:
+                debian_context['changelog'] = overall_result.changelog_behaviour.json()
             with open(os.environ['SVP_RESULT'], 'w') as f:
                 json.dump({
                     'value': calculate_value(all_fixed_lintian_tags),
+                    'debian': debian_context,
                     'context': {
                         'applied': applied,
                         'failed': failed,
