@@ -34,7 +34,7 @@ import shutil
 import subprocess
 import sys
 from tempfile import TemporaryDirectory
-from typing import Optional, Tuple, List, Dict
+from typing import Optional, Tuple, List, Dict, Any
 from urllib.parse import urlparse
 
 
@@ -45,25 +45,38 @@ from debian.deb822 import PkgRelation
 from breezy import osutils
 from breezy.branch import Branch
 from breezy.controldir import ControlDir
-from breezy.errors import AlreadyBranchError, NotBranchError, FileExists
+from breezy.errors import AlreadyBranchError, NotBranchError, FileExists, NoSuchRevision
 from breezy.commit import NullCommitReporter, PointlessCommit
 from breezy.revision import NULL_REVISION
 from breezy.workingtree import WorkingTree
 
 from ognibuild import DetailedFailure, UnidentifiedError
+from ognibuild.buildlog import InstallFixer, problem_to_upstream_requirement
 from ognibuild.buildsystem import get_buildsystem, NoBuildToolsFound
 from ognibuild.debian.apt import AptManager
 from ognibuild.debian.build import DEFAULT_BUILDER
+from ognibuild.debian.fix_build import (
+    DetailedDebianBuildFailure,
+    UnidentifiedDebianBuildError,
+    build_incrementally,
+    )
 from ognibuild.dist import (  # noqa: F401
     DistNoTarball,
     create_dist as ogni_create_dist,
     )
+from ognibuild.fix_build import iterate_with_build_fixers, BuildFixer
+
+from ognibuild.session import SessionSetupFailure
 from ognibuild.session.plain import PlainSession
 from ognibuild.session.schroot import SchrootSession
-from ognibuild.requirements import CargoCrateRequirement, Requirement
-from ognibuild.resolver.apt import AptResolver
+from ognibuild.requirements import (
+    Requirement,
+    CargoCrateRequirement,
+    PythonPackageRequirement,
+    GoPackageRequirement,
+    )
+from ognibuild.resolver.apt import AptResolver, AptRequirement
 from ognibuild.vcs import dupe_vcs_tree
-from ognibuild.buildlog import InstallFixer, problem_to_upstream_requirement
 
 from upstream_ontologist.guess import (
     guess_upstream_info,
@@ -75,6 +88,13 @@ from upstream_ontologist.debian import (
     valid_debian_package_name,
 )
 
+from breezy.plugins.debian import default_orig_dir
+from breezy.plugins.debian.merge_upstream import (
+    get_tarballs,
+    do_import,
+    get_existing_imported_upstream_revids,
+    )
+from breezy.plugins.debian.import_dsc import UpstreamAlreadyImported
 from breezy.plugins.debian.upstream import PackageVersionNotPresent
 from breezy.plugins.debian.upstream.pristinetar import get_pristine_tar_source
 from breezy.plugins.debian.upstream.branch import (
@@ -83,8 +103,13 @@ from breezy.plugins.debian.upstream.branch import (
     DistCommandFailed,
     run_dist_command,
 )
+from breezy.workspace import (
+    check_clean_tree,
+    reset_tree,
+    WorkspaceDirty,
+    )
 
-from buildlog_consultant.common import VcsControlDirectoryNeeded
+from buildlog_consultant.common import VcsControlDirectoryNeeded, SetuptoolScmVersionIssue
 
 from debmutate.versions import (
     debianize_upstream_version,
@@ -94,13 +119,11 @@ from debmutate.versions import (
 from . import (
     available_lintian_fixers,
     version_string,
-    check_clean_tree,
-    PendingChanges,
     run_lintian_fixers,
     get_committer,
-    reset_tree,
     version_string as lintian_brush_version_string,
 )
+from .debbugs import find_archived_wnpp_bugs, find_wnpp_bugs
 from .debhelper import (
     maximum_debhelper_compat_version,
     write_rules_template as write_debhelper_rules_template,
@@ -171,36 +194,6 @@ def write_changelog_template(path, source_name, version, author=None, wnpp_bugs=
         f.write(cl.__str__().strip("\n") + "\n")
 
 
-async def find_archived_wnpp_bugs(source_name):
-    try:
-        from .udd import connect_udd_mirror
-    except ModuleNotFoundError:
-        logging.warning("asyncpg not available, unable to find wnpp bugs.")
-        return []
-    async with await connect_udd_mirror() as conn:
-        return [
-            (row[0], row[1])
-            for row in await conn.fetch("""\
-select id, substring(title, 0, 3) from archived_bugs where package = 'wnpp' and
-title like 'ITP: ' || $1 || ' -- %' OR
-title like 'RFP: ' || $1 || ' -- %'
-""", source_name)]
-
-
-async def find_wnpp_bugs(source_name):
-    try:
-        from .udd import connect_udd_mirror
-    except ModuleNotFoundError:
-        logging.warning("asyncpg not available, unable to find wnpp bugs.")
-        return []
-    async with await connect_udd_mirror() as conn:
-        return [
-            (row[0], row['type'])
-            for row in await conn.fetch("""\
-select id, type from wnpp where source = $1 and type in ('ITP', 'RFP')
-""", source_name)]
-
-
 MINIMUM_CERTAINTY = "possible"  # For now..
 
 
@@ -249,7 +242,9 @@ def default_create_dist(session, tree, package, version, target_dir):
                     subdir=(package or "package"),
                     cleanup=False)
             except DetailedFailure as e:
-                if isinstance(e.error, VcsControlDirectoryNeeded):
+                if isinstance(
+                        e.error,
+                        (VcsControlDirectoryNeeded, SetuptoolScmVersionIssue)):
                     return ogni_create_dist(
                         session, tree, target_dir,
                         include_controldir=True,
@@ -266,6 +261,11 @@ def default_create_dist(session, tree, package, version, target_dir):
             "Build system does not support dist, falling back "
             "to export.")
         return None
+    except SessionSetupFailure as e:
+        raise DistCommandFailed(str(e), 'session-setup-failure')
+    except DistNoTarball as e:
+        logging.info("Build system did not create a tarball: %s", e)
+        return None
     except DetailedFailure as e:
         raise DistCommandFailed(str(e), e.error)
     except UnidentifiedError as e:
@@ -275,14 +275,6 @@ def default_create_dist(session, tree, package, version, target_dir):
 def import_upstream_version_from_dist(
         wt, subpath, upstream_source, source_name, upstream_version,
         session):
-    from breezy.plugins.debian import default_orig_dir
-    from breezy.plugins.debian.merge_upstream import (
-        get_tarballs,
-        do_import,
-        get_existing_imported_upstream_revids,
-        )
-    from breezy.plugins.debian.import_dsc import UpstreamAlreadyImported
-
     orig_dir = os.path.abspath(default_orig_dir)
 
     tag_names = {}
@@ -319,12 +311,11 @@ def import_upstream_version_from_dist(
 
     upstream_branch_name = "upstream"
     try:
-        wt.controldir.create_branch(upstream_branch_name).generate_revision_history(
-            pristine_revids[None]
-        )
+        branch = wt.controldir.create_branch(upstream_branch_name)
     except AlreadyBranchError:
         logging.info("Upstream branch already exists; not creating.")
     else:
+        branch.generate_revision_history(pristine_revids[None])
         logging.info("Created upstream branch.")
 
     return pristine_revids, tag_names, upstream_branch_name
@@ -637,18 +628,24 @@ def process_cargo(es, session, wt, subpath, debian_path, upstream_version, metad
     wt.branch.generate_revision_history(NULL_REVISION)
     reset_tree(wt, wt.basis_tree(), subpath)
     from debmutate.debcargo import DebcargoControlShimEditor
-    upstream_name = metadata['Name'].replace('_', '-')
-    control = es.enter_context(DebcargoControlShimEditor.from_debian_dir(wt.abspath(debian_path), upstream_name, upstream_version))
+    crate = metadata.get('X-Cargo-Crate')
+    if crate is None:
+        crate = metadata['Name'].replace('_', '-')
+    if not wt.has_filename(debian_path):
+        wt.mkdir(debian_path)
+    if not wt.is_versioned(debian_path):
+        wt.add(debian_path)
+    control = es.enter_context(DebcargoControlShimEditor.from_debian_dir(wt.abspath(debian_path), crate, upstream_version))
     # Only set semver_suffix if this is not the latest version
     import semver
     try:
         desired_version = semver.VersionInfo.parse(upstream_version)
     except ValueError as e:
         raise BuildSystemProcessError(buildsystem, str(e), e)
-    data = load_crate_info(upstream_name)
+    data = load_crate_info(crate)
     if data is None:
         raise BuildSystemProcessError(
-            buildsystem, 'Crate does not exist' % upstream_name)
+            buildsystem, 'Crate does not exist' % crate)
     for version_info in data['versions']:
         available_version = semver.VersionInfo.parse(version_info['num'])
         if (available_version.major, available_version.minor) > (desired_version.major, desired_version.minor):
@@ -740,7 +737,7 @@ def import_build_deps(source, build_deps):
 def import_upstream_dist(
         pristine_tar_source, wt, upstream_source, subpath, source_name,
         upstream_version, session):
-    if pristine_tar_source.has_version(source_name, upstream_version):
+    if pristine_tar_source.has_version(source_name, upstream_version, try_hard=False):
         logging.warning(
             'Upstream version %s/%s already imported.',
             source_name, upstream_version)
@@ -776,7 +773,7 @@ def generic_get_source_name(wt, metadata):
 
 def get_upstream_version(
         upstream_source, metadata, local_dir=None,
-        upstream_version=None):
+        upstream_version: Optional[str] = None) -> Tuple[str, str]:
     if upstream_version is None:
         upstream_version, mangled_upstream_version = upstream_source.get_latest_version(metadata.get("Name"), None)
     else:
@@ -805,17 +802,12 @@ def get_upstream_version(
     return upstream_version, mangled_upstream_version
 
 
-def find_wnpp_bugs_harder(source_name, upstream_name):
-    import asyncio
-
-    loop = asyncio.get_event_loop()
-    wnpp_bugs = loop.run_until_complete(find_wnpp_bugs(source_name))
+async def find_wnpp_bugs_harder(source_name, upstream_name):
+    wnpp_bugs = await find_wnpp_bugs(source_name)
     if not wnpp_bugs and source_name != upstream_name:
-        wnpp_bugs = loop.run_until_complete(find_wnpp_bugs(upstream_name))
+        wnpp_bugs = await find_wnpp_bugs(upstream_name)
     if not wnpp_bugs:
-        wnpp_bugs = loop.run_until_complete(
-            find_archived_wnpp_bugs(source_name)
-        )
+        wnpp_bugs = await find_archived_wnpp_bugs(source_name)
         if wnpp_bugs:
             logging.warning(
                 "Found archived ITP/RFP bugs for %s: %r", source_name, [bug for bug, kind in wnpp_bugs]
@@ -831,7 +823,7 @@ def find_wnpp_bugs_harder(source_name, upstream_name):
 
 def debianize(  # noqa: C901
     wt: WorkingTree, subpath: str,
-    upstream_branch: Branch, upstream_subpath: Optional[str],
+    upstream_branch: Optional[Branch], upstream_subpath: Optional[str],
     use_inotify: Optional[bool] = None,
     diligence: int = 0,
     trust: bool = False,
@@ -850,7 +842,9 @@ def debianize(  # noqa: C901
     debian_revision: str = "1",
     upstream_version: Optional[str] = None,
     requirement: Optional[Requirement] = None,
-    team: Optional[str] = None
+    team: Optional[str] = None,
+    buildsystem_name: Optional[str] = None,
+    metadata: Optional[Dict[str, Any]] = None
 ):
     if committer is None:
         committer = get_committer(wt)
@@ -861,7 +855,10 @@ def debianize(  # noqa: C901
             raise DebianDirectoryExists(wt.abspath(subpath))
 
     metadata_items = []
-    metadata = {}
+    if metadata is None:
+        metadata = {}
+    else:
+        metadata = dict(metadata)
 
     def import_metadata_from_path(p):
         metadata_items.extend(guess_upstream_info(p, trust_package=trust))
@@ -884,6 +881,8 @@ def debianize(  # noqa: C901
 
             if not wt.has_filename(debian_path):
                 wt.mkdir(debian_path)
+            if not wt.is_versioned(debian_path):
+                wt.add(debian_path)
 
             if schroot is None:
                 session = PlainSession()
@@ -891,20 +890,28 @@ def debianize(  # noqa: C901
                 logging.info('Using schroot %s', schroot)
                 session = SchrootSession(schroot)
 
-            upstream_source = UpstreamBranchSource.from_branch(
-                upstream_branch, version_kind=upstream_version_kind, local_dir=wt.controldir,
-                create_dist=(create_dist or partial(default_create_dist, session)))
+            if upstream_branch:
+                upstream_source = UpstreamBranchSource.from_branch(
+                    upstream_branch, version_kind=upstream_version_kind, local_dir=wt.controldir,
+                    create_dist=(create_dist or partial(default_create_dist, session)))
+            else:
+                upstream_source = None
 
-            upstream_version, mangled_upstream_version = get_upstream_version(
-                upstream_source, metadata, local_dir=wt.controldir,
-                upstream_version=upstream_version)
+            if upstream_version is not None:
+                mangled_upstream_version = debianize_upstream_version(upstream_version)
+            else:
+                upstream_version, mangled_upstream_version = get_upstream_version(
+                    upstream_source, metadata, local_dir=wt.controldir,
+                    upstream_version=upstream_version)
 
             result.upstream_version = upstream_version
 
             source_name = generic_get_source_name(wt, metadata)
 
             def kickstart_from_dist(wt, subpath):
-                logging.info("Using upstream version %s", mangled_upstream_version)
+                logging.info(
+                    "Kickstarting from dist tarball. Using upstream version %s",
+                    mangled_upstream_version)
 
                 pristine_tar_source = get_pristine_tar_source(wt, wt.branch)
                 upstream_dist_revid, result.upstream_branch_name, result.tag_names = import_upstream_dist(
@@ -922,7 +929,7 @@ def debianize(  # noqa: C901
 
                 if wt.has_filename(debian_path) and force_new_directory:
                     shutil.rmtree(wt.abspath(debian_path))
-                    os.mkdir(wt.abspath(debian_path))
+                    wt.mkdir(wt.abspath(debian_path))
                     try:
                         wt.commit(
                             'Remove old debian directory', specific_files=[debian_path],
@@ -930,18 +937,40 @@ def debianize(  # noqa: C901
                     except PointlessCommit:
                         pass
 
-            upstream_vcs_tree = upstream_source.revision_tree(source_name, mangled_upstream_version)
+                wt.mkdir(os.path.join(debian_path, 'source'))
+                wt.add(os.path.join(debian_path, 'source'))
+                wt.put_file_bytes_non_atomic(
+                    os.path.join(debian_path, 'source', 'format'), b'3.0 (quilt)\n')
 
-            # TODO(jelmer): Don't export, just access from memory.
-            exported_upstream_tree_path = es.enter_context(TemporaryDirectory())
-            dupe_vcs_tree(upstream_vcs_tree, exported_upstream_tree_path)
-            import_metadata_from_path(exported_upstream_tree_path)
-            buildsystem_subpath, buildsystem = get_buildsystem(exported_upstream_tree_path)
-
-            if buildsystem:
+            if upstream_source:
                 try:
-                    process = PROCESSORS[buildsystem.name]
+                    upstream_vcs_tree = upstream_source.revision_tree(source_name, mangled_upstream_version)
+                except (PackageVersionNotPresent, NoSuchRevision):
+                    logging.warning(
+                        'Unable to find upstream version %s/%s in upstream source %r. '
+                        'Unable to extract metadata.',
+                        source_name, mangled_upstream_version, upstream_source)
+                else:
+                    # TODO(jelmer): Don't export, just access from memory.
+                    exported_upstream_tree_path = es.enter_context(TemporaryDirectory())
+                    dupe_vcs_tree(upstream_vcs_tree, exported_upstream_tree_path)
+                    import_metadata_from_path(exported_upstream_tree_path)
+
+            if buildsystem_name is None:
+                buildsystem_subpath, buildsystem = get_buildsystem(exported_upstream_tree_path)
+                if buildsystem:
+                    buildsystem_name = buildsystem.name
+            else:
+                buildsystem = None
+                buildsystem_subpath = ''
+
+            if buildsystem_name:
+                try:
+                    process = PROCESSORS[buildsystem_name]
                 except KeyError:
+                    logging.warning(
+                        'No support in debianize for build system %s, falling back to default.',
+                        buildsystem_name)
                     process = process_default
             else:
                 process = process_default
@@ -972,7 +1001,12 @@ def debianize(  # noqa: C901
                 raise SourcePackageNameInvalid(source['Source'])
 
             if net_access:
-                wnpp_bugs = find_wnpp_bugs_harder(source['Source'], metadata.get('Name'))
+                import asyncio
+
+                loop = asyncio.get_event_loop()
+
+                wnpp_bugs = loop.run_until_complete(
+                    find_wnpp_bugs_harder(source['Source'], metadata.get('Name')))
             else:
                 wnpp_bugs = None
 
@@ -1049,14 +1083,17 @@ def debianize(  # noqa: C901
 @dataclass
 class UpstreamInfo:
     name: Optional[str]
+    buildsystem: Optional[str] = None
     branch_url: Optional[str] = None
     branch_subpath: Optional[str] = None
     tarball_url: Optional[str] = None
     version: Optional[str] = None
+    metadata: Dict[str, Any] = field(default_factory=dict)
 
     def json(self):
         return {
             'name': self.name,
+            'buildsystem': self.buildsystem,
             'branch_url': self.branch_url,
             'branch_subpath': self.branch_subpath,
             'tarball_url': self.tarball_url,
@@ -1126,10 +1163,10 @@ def find_cargo_crate_upstream(requirement):
     upstream_branch = data['crate']['repository']
     name = 'rust-' + data['crate']['name'].replace('_', '-')
     version = None
-    if requirement.version is not None:
+    if requirement.api_version is not None:
         for version_info in data['versions']:
-            if (not version_info['num'].startswith(requirement.version + '.') and
-                    not version_info['num'] == requirement.version):
+            if (not version_info['num'].startswith(requirement.api_version + '.') and
+                    not version_info['num'] == requirement.api_version):
                 continue
             if version is None:
                 version = semver.VersionInfo.parse(version_info['num'])
@@ -1137,31 +1174,93 @@ def find_cargo_crate_upstream(requirement):
                 version = semver.max_ver(version, semver.VersionInfo.parse(version_info['num']))
         if version is None:
             logging.warning(
-                'Unable to find version of crate %s that matches version %s',
-                name, requirement.version)
+                'Unable to find version of crate %s that matches API version %s',
+                name, requirement.api_version)
         else:
             name += '-' + semver_pair(str(version))
     return UpstreamInfo(
         branch_url=upstream_branch, branch_subpath=None,
-        name=name, version=str(version) if version else None)
+        name=name, version=str(version) if version else None,
+        metadata={'X-Cargo-Crate': data['crate']['name']},
+        buildsystem='cargo')
 
 
-def find_apt_upstream(requirement):
+def apt_to_cargo_requirement(m, rels):
+    name = m.group(1)
+    api_version = m.group(2)
+    if m.group(3):
+        features = set(m.group(3)[1:].split('-'))
+    else:
+        features = set()
+    if not rels:
+        minimum_version = None
+    elif len(rels) == 1 and rels[0][0] == '>=':
+        minimum_version = Version(rels[0][1]).upstream_version
+    else:
+        logging.warning('Unable to parse Debian version %r', rels)
+        minimum_version = None
+
+    return CargoCrateRequirement(
+        name, api_version=api_version,
+        features=features, minimum_version=minimum_version)
+
+
+def apt_to_python_requirement(m, rels):
+    name = m.group(2)
+    python_version = m.group(1)
+    if not rels:
+        minimum_version = None
+    elif len(rels) == 1 and rels[0][0] == '>=':
+        minimum_version = Version(rels[0][1]).upstream_version
+    else:
+        logging.warning('Unable to parse Debian version %r', rels)
+        minimum_version = None
+    return PythonPackageRequirement(
+        name, python_version=(python_version or None),
+        minimum_version=minimum_version)
+
+
+def apt_to_go_requirement(m, rels):
+    parts = m.group(1).split('-')
+    if parts[0] == 'github':
+        parts[0] = 'github.com'
+    if parts[0] == 'gopkg':
+        parts[0] = 'gopkg.in'
+    if not rels:
+        version = None
+    elif len(rels) == 1 and rels[0][0] == '=':
+        version = Version(rels[0][1]).upstream_version
+    else:
+        logging.warning('Unable to parse Debian version %r', rels)
+        version = None
+    return GoPackageRequirement('/'.join(parts), version=version)
+
+
+BINARY_PACKAGE_UPSTREAM_MATCHERS = [
+    (r'librust-(.*)-([^-+]+)(\+.*?)-dev', apt_to_cargo_requirement),
+    (r'python([0-9.]*)-(.*)', apt_to_python_requirement),
+    (r'golang-(.*)-dev', apt_to_go_requirement),
+]
+
+
+_BINARY_PACKAGE_UPSTREAM_MATCHERS = [
+    (re.compile(r), fn) for (r, fn) in BINARY_PACKAGE_UPSTREAM_MATCHERS]
+
+
+def find_apt_upstream(requirement: AptRequirement) -> Optional[UpstreamInfo]:
     for option in requirement.relations:
         for rel in option:
-            m = re.match(r'librust-(.*)-([^-+]+)(\+.*?)-dev', rel['name'])
-            if m:
-                name = m.group(1)
-                version = m.group(2)
-                if m.group(3):
-                    features = set(m.group(3)[1:].split('-'))
-                else:
-                    features = set()
-                return find_upstream(
-                    CargoCrateRequirement(name, version=version, features=features))
+            for matcher, fn in _BINARY_PACKAGE_UPSTREAM_MATCHERS:
+                m = matcher.fullmatch(rel['name'])
+                if m:
+                    upstream_requirement = fn(m, [rel['version']] if rel['version'] else [])
+                    return find_upstream(upstream_requirement)
+
+            logging.warning('Unable to map binary package name %s to upstream', rel['name'])
+    return None
 
 
-def find_or_upstream(requirement) -> Optional[UpstreamInfo]:
+def find_or_upstream(requirement: Requirement) -> Optional[UpstreamInfo]:
     for req in requirement.elements:
         info = find_upstream(req)
         if info is not None:
@@ -1178,7 +1277,7 @@ UPSTREAM_FINDER = {
     }
 
 
-def find_upstream(requirement) -> Optional[UpstreamInfo]:  # noqa: C901
+def find_upstream(requirement: Requirement) -> Optional[UpstreamInfo]:  # noqa: C901
     try:
         return UPSTREAM_FINDER[requirement.family](requirement)
     except KeyError:
@@ -1232,8 +1331,7 @@ class SimpleTrustedAptRepo(object):
             with httpd:  # to make sure httpd.server_close is called
                 httpd.serve_forever()
 
-        self.thread = Thread(target=serve_forever, args=(self.httpd, ))
-        self.thread.setDaemon(True)
+        self.thread = Thread(target=serve_forever, args=(self.httpd, ), daemon=True)
         self.thread.start()
 
     def stop(self):
@@ -1249,6 +1347,137 @@ class SimpleTrustedAptRepo(object):
             f.write(packages)
 
 
+def use_packaging_branch(wt: WorkingTree, branch_name: str) -> None:
+    last_revision = wt.last_revision()
+    try:
+        target_branch = wt.controldir.open_branch(branch_name)
+    except NotBranchError:
+        target_branch = wt.controldir.create_branch(branch_name)
+
+    target_branch.generate_revision_history(last_revision)
+    logging.info('Switching to packaging branch %s.', branch_name)
+    wt.controldir.set_branch_reference(target_branch, name="")
+    # TODO(jelmer): breezy bug?
+    wt._branch = target_branch
+
+
+class DebianizeFixer(BuildFixer):
+
+    def __str__(self):
+        return "debianize fixer"
+
+    def __repr__(self):
+        return "%s(%r, %r)" % (
+            type(self).__name__, self.vcs_directory,
+            self.apt_repo)
+
+    def __init__(self, vcs_directory, apt_repo, do_build, diligence=0,
+                 trust=False, check=True, net_access=True,
+                 force_new_directory=False,
+                 team=None, verbose=False, force_subprocess=False,
+                 upstream_version_kind="release",
+                 debian_revision="1",
+                 schroot=None,
+                 consult_external_directory=True,
+                 use_inotify=None,
+                 create_dist=None,
+                 compat_release=None):
+        self.vcs_directory = vcs_directory
+        self.apt_repo = apt_repo
+        self.diligence = diligence
+        self.trust = trust
+        self.check = check
+        self.net_access = net_access
+        self.force_new_directory = force_new_directory
+        self.team = team
+        self.verbose = verbose
+        self.force_subprocess = force_subprocess
+        self.upstream_version_kind = upstream_version_kind
+        self.debian_revision = debian_revision
+        self.consult_external_directory = consult_external_directory
+        self.schroot = schroot
+        self.use_inotify = use_inotify
+        self.create_dist = create_dist
+        self.compat_release = compat_release
+        self.do_build = do_build
+
+    def can_fix(self, problem):
+        requirement = problem_to_upstream_requirement(problem)
+        if requirement is None:
+            return False
+        return find_upstream(requirement) is not None
+
+    def _fix(self, problem, phase: Tuple[str, ...]):
+        requirement = problem_to_upstream_requirement(problem)
+        if requirement is None:
+            return False
+        logging.debug(
+            'Translated problem %r to requirement %r', problem, requirement)
+        upstream_info = find_upstream(requirement)
+        if upstream_info is None:
+            logging.error(
+                'Unable to find upstream information for requirement %r',
+                requirement)
+            return False
+        if upstream_info.branch_url:
+            logging.info(
+                'Packaging %r to address %r',
+                upstream_info.branch_url, problem)
+            try:
+                upstream_branch = Branch.open(upstream_info.branch_url)
+            except NotBranchError as e:
+                logging.warning(
+                    'Unable to open branch %s: %r', upstream_info.branch_url,
+                    e)
+                upstream_branch = None
+        else:
+            upstream_branch = None
+        if upstream_info.name is not None:
+            vcs_path = os.path.join(
+                self.vcs_directory,
+                upstream_info.name.replace('/', '-'))
+        else:
+            raise AssertionError('no upstream name provided')
+        if os.path.exists(vcs_path):
+            shutil.rmtree(vcs_path)
+        if upstream_branch:
+            format = upstream_branch.controldir.cloning_metadir()
+        else:
+            # TODO(jelmer): default to git?
+            format = None
+        result = ControlDir.create_branch_convenience(
+            vcs_path, force_new_tree=True,
+            format=format)
+        new_wt = result.controldir.open_workingtree()
+        new_subpath = ''
+        debianize(
+            new_wt, new_subpath,
+            upstream_branch, upstream_info.branch_subpath or '',
+            use_inotify=self.use_inotify,
+            diligence=self.diligence,
+            create_dist=self.create_dist,
+            trust=self.trust,
+            check=self.check,
+            net_access=self.net_access,
+            force_new_directory=self.force_new_directory,
+            force_subprocess=self.force_subprocess,
+            compat_release=self.compat_release,
+            consult_external_directory=self.consult_external_directory,
+            verbose=self.verbose, schroot=self.schroot,
+            debian_revision=self.debian_revision,
+            upstream_version=upstream_info.version,
+            upstream_version_kind=self.upstream_version_kind,
+            requirement=requirement,
+            buildsystem_name=upstream_info.buildsystem,
+            team=self.team,
+            metadata=upstream_info.metadata)
+        self.do_build(
+            new_wt, new_subpath, self.apt_repo.directory,
+            extra_repositories=self.apt_repo.sources_lines())
+        self.apt_repo.refresh()
+        return True
+
+
 def report_fatal(code, description, hint=None, details=None):
     if os.environ.get('SVP_API') == '1':
         with open(os.environ['SVP_RESULT'], 'w') as f:
@@ -1377,6 +1606,16 @@ def main(argv=None):  # noqa: C901
     parser.add_argument(
         '--team', type=str,
         help='Maintainer team ("$NAME <$EMAIL>")')
+    parser.add_argument(
+        '--debian-branch', type=str,
+        help=('Name of Debian branch to create. Empty string to stay at '
+              'current branch.'),
+        default='%(vendor)s/main')
+    parser.add_argument(
+        '--debian-binary', type=str,
+        help=(
+            'Package whatever source will create the named Debian '
+            'binary package.'))
     parser.add_argument('upstream', nargs='?', type=str)
 
     args = parser.parse_args(argv)
@@ -1416,10 +1655,26 @@ def main(argv=None):  # noqa: C901
         except NotBranchError as e:
             logging.fatal('%s: not a valid branch: %s', args.upstream, e)
             return 1
+    elif args.debian_binary:
+        apt_requirement = AptRequirement.from_str(args.debian_binary)
+        upstream_info = find_apt_upstream(apt_requirement)
+        if not upstream_info:
+            logging.fatal(
+                '%s: Unable to find upstream info for %s', args.debian_binary,
+                apt_requirement)
+            return 1
+        logging.info('Found relevant upstream branch at %s', upstream_info.branch_url)
+        upstream_branch = Branch.open(upstream_info.branch_url)
+        upstream_subpath = upstream_info.branch_subpath
     else:
         upstream_branch = wt.branch
         upstream_subpath = subpath
 
+    if args.debian_branch:
+        from debmutate.vendor import get_vendor_name
+
+        use_packaging_branch(wt, args.debian_branch % {'vendor': get_vendor_name().lower()})
+
     use_inotify = ((False if args.disable_inotify else None),)
     with wt.lock_write():
         try:
@@ -1454,7 +1709,7 @@ def main(argv=None):  # noqa: C901
         except DistCommandFailed as e:
             report_fatal(e.kind or "dist-command-failed", e.error)
             return 1
-        except PendingChanges:
+        except WorkspaceDirty:
             report_fatal("pending-changes", "Please commit pending changes first.")
             return 1
         except DebianDirectoryExists as e:
@@ -1484,6 +1739,11 @@ def main(argv=None):  # noqa: C901
                 str(e),
                 details=(e.error.json() if e.error else None))
             return 1
+        except BuildSystemProcessError as e:
+            report_fatal(
+                'build-system-process-error',
+                e.message)
+            return 1
         except OSError as e:
             if e.errno == errno.ENOSPC:
                 report_fatal('no-space-on-device', str(e))
@@ -1495,87 +1755,6 @@ def main(argv=None):  # noqa: C901
         args.iterate_fix = True
 
     if args.iterate_fix:
-        from ognibuild.fix_build import iterate_with_build_fixers, BuildFixer
-        from ognibuild.debian.fix_build import (
-            DetailedDebianBuildFailure,
-            UnidentifiedDebianBuildError,
-            build_incrementally,
-            )
-
-        class DebianizeFixer(BuildFixer):
-
-            def __str__(self):
-                return "debianize fixer"
-
-            def __repr__(self):
-                return "%s(%r, %r)" % (
-                    type(self).__name__, self.vcs_directory,
-                    self.apt_repo)
-
-            def __init__(self, vcs_directory, apt_repo):
-                self.vcs_directory = vcs_directory
-                self.apt_repo = apt_repo
-
-            def can_fix(self, problem):
-                requirement = problem_to_upstream_requirement(problem)
-                if requirement is None:
-                    return False
-                return find_upstream(requirement) is not None
-
-            def _fix(self, problem, phase: Tuple[str, ...]):
-                requirement = problem_to_upstream_requirement(problem)
-                if requirement is None:
-                    return False
-                logging.debug(
-                    'Translated problem %r to requirement %r', problem, requirement)
-                upstream_info = find_upstream(requirement)
-                if upstream_info is None:
-                    logging.error(
-                        'Unable to find upstream information for requirement %r',
-                        requirement)
-                    return False
-                logging.info(
-                    'Packaging %r to address %r',
-                    upstream_info.branch_url, problem)
-                upstream_branch = Branch.open(upstream_info.branch_url)
-                if upstream_info.name is not None:
-                    vcs_path = os.path.join(
-                        self.vcs_directory,
-                        upstream_info.name.replace('/', '-'))
-                else:
-                    raise AssertionError('no upstream name provided')
-                if os.path.exists(vcs_path):
-                    shutil.rmtree(vcs_path)
-                result = ControlDir.create_branch_convenience(
-                    vcs_path, force_new_tree=True,
-                    format=upstream_branch.controldir.cloning_metadir())
-                new_wt = result.controldir.open_workingtree()
-                new_subpath = ''
-                debianize(
-                    new_wt, new_subpath,
-                    upstream_branch, upstream_info.branch_subpath or '',
-                    use_inotify=use_inotify,
-                    diligence=args.diligence,
-                    create_dist=create_dist,
-                    trust=args.trust,
-                    check=args.check,
-                    net_access=not args.disable_net_access,
-                    force_new_directory=args.force_new_directory,
-                    force_subprocess=args.force_subprocess,
-                    compat_release=compat_release,
-                    consult_external_directory=args.consult_external_directory,
-                    verbose=args.verbose, schroot=args.schroot,
-                    debian_revision=args.debian_revision,
-                    upstream_version=upstream_info.version,
-                    upstream_version_kind=args.upstream_version_kind,
-                    requirement=requirement,
-                    team=args.team)
-                do_build(
-                    new_wt, new_subpath, self.apt_repo.directory,
-                    extra_repositories=self.apt_repo.sources_lines())
-                self.apt_repo.refresh()
-                return True
-
         if args.schroot is None:
             session = PlainSession()
         else:
@@ -1623,7 +1802,21 @@ def main(argv=None):  # noqa: C901
                             extra_repositories=apt_repo.sources_lines())
                     with SimpleTrustedAptRepo(apt_directory) as apt_repo:
                         (changes_names, cl_entry) = iterate_with_build_fixers(
-                            [DebianizeFixer(vcs_directory, apt_repo)],
+                            [DebianizeFixer(
+                                vcs_directory, apt_repo,
+                                diligence=args.diligence,
+                                trust=args.trust,
+                                check=args.check,
+                                net_access=not args.disable_net_access,
+                                force_new_directory=args.force_new_directory,
+                                team=args.team, verbose=args.verbose,
+                                force_subprocess=args.force_subprocess,
+                                upstream_version_kind=args.upstream_version_kind,
+                                debian_revision=args.debian_revision,
+                                schroot=args.schroot, use_inotify=use_inotify,
+                                consult_external_directory=args.consult_external_directory,
+                                create_dist=create_dist, compat_release=compat_release,
+                                do_build=do_build)],
                             main_build)
                 else:
                     (changes_names, cl_entry) = do_build(
