@@ -19,7 +19,7 @@
 
 from collections.abc import MutableMapping
 from debmutate.control import ControlEditor
-from debmutate.deb822 import Deb822
+from debmutate.vendor import get_vendor_name
 import sys
 from typing import Optional, Tuple, Union, List
 
@@ -101,7 +101,7 @@ class LintianIssue(object):
 
 _fixed_lintian_issues: List[LintianIssue] = []
 _present_overrides: Optional[List[LintianOverride]] = None
-_overriden_issues = []
+_overriden_issues: List[LintianIssue] = []
 _tag_renames = None
 
 
@@ -222,15 +222,6 @@ def source_package_name():
     return os.environ.get("DEB_SOURCE")
 
 
-def vendor():
-    try:
-        return os.environ['DEB_VENDOR']
-    except KeyError:
-        with open('/etc/dpkg/origins/default', 'r') as f:
-            c = Deb822(f)
-            return c['Vendor']
-
-
 def is_debcargo_package():
     return os.path.exists('debian/debcargo.toml')
 
@@ -244,3 +235,7 @@ if is_debcargo_package():
         control = DebcargoControlShimEditor(DebcargoEditor())
 else:
     control = ControlEditor()
+
+
+def vendor() -> str:
+    return get_vendor_name()
